%bcond check 1

Name:           uv
Version:        0.2.13
Release:        %autorelease
Summary:        An extremely fast Python package installer and resolver, written in Rust

# The license of the uv project is (MIT OR Apache-2.0); Rust crates compiled
# into the executable contribute additional license terms. To obtain the
# following list of licenses, build the package and note the output of
# %%{cargo_license_summary}.
#
# (Apache-2.0 OR MIT) AND BSD-3-Clause
# (MIT OR Apache-2.0) AND Unicode-DFS-2016
# 0BSD
# 0BSD OR MIT OR Apache-2.0
# Apache-2.0
# Apache-2.0 OR BSD-2-Clause
# Apache-2.0 OR BSL-1.0
# Apache-2.0 OR ISC OR MIT
# Apache-2.0 OR MIT
# Apache-2.0 WITH LLVM-exception
# Apache-2.0 WITH LLVM-exception OR Apache-2.0 OR MIT
# BSD-3-Clause
# BSD-3-Clause AND MIT
# ISC
# ISC AND MIT AND OpenSSL
# LGPL-3.0-or-later OR MPL-2.0
# MIT
# MIT OR Apache-2.0
# MIT OR Apache-2.0 OR Zlib
# MIT OR LGPL-3.0-or-later
# MIT OR Zlib OR Apache-2.0
# MIT-0 OR Apache-2.0
# MPL-2.0
# Unlicense OR MIT
# Zlib OR Apache-2.0 OR MIT
License:        %{shrink:
                0BSD AND
                (0BSD OR Apache-2.0 OR MIT) AND
                Apache-2.0 AND
                (Apache-2.0 OR BSD-2-Clause) AND
                (Apache-2.0 OR BSL-1.0) AND
                (Apache-2.0 OR ISC OR MIT) AND
                (Apache-2.0 OR MIT) AND
                (Apache-2.0 OR MIT OR Zlib) AND
                (Apache-2.0 OR MIT-0) AND
                (Apache-2.0 WITH LLVM-exception) AND
                (Apache-2.0 WITH LLVM-exception OR Apache-2.0 OR MIT) AND
                BSD-3-Clause AND
                ISC AND
                (LGPL-3.0-or-later OR MIT) AND
                (LGPL-3.0-or-later OR MPL-2.0) AND
                MIT AND
                (MIT OR Unlicense) AND
                MPL-2.0 AND
                OpenSSL AND
                Unicode-DFS-2016
                }
# LICENSE.dependencies contains a full license breakdown
URL:            https://github.com/astral-sh/uv
Source0:        %{url}/archive/%{version}/uv-%{version}.tar.gz

# Currently, uv must use a fork of async_zip, as explained in:
#   Restore central directory buffering
#   https://github.com/charliermarsh/rs-async-zip/pull/2
# and further discussed in
#   Please consider supporting the current release of async_zip
#   https://github.com/prefix-dev/async_http_range_reader/issues/14
# We therefore bundle the fork as prescribed in
#   https://docs.fedoraproject.org/en-US/packaging-guidelines/Rust/#_replacing_git_dependencies
%global async_zip_git https://github.com/charliermarsh/rs-async-zip
%global async_zip_rev 1dcb40cfe1bf5325a6fd4bfcf9894db40241f585
%global async_zip_baseversion 0.0.17
%global async_zip_snapdate 20240403
Source1:        %{async_zip_git}/archive/%{async_zip_rev}/rs-async-zip-%{async_zip_rev}.tar.gz

# For the foreseeable future, uv must use a fork of pubgrub, as explained in:
#   Plans for eventually using published pubgrub?
#   https://github.com/astral-sh/uv/issues/3794
# We therefore bundle the fork as prescribed in
#   https://docs.fedoraproject.org/en-US/packaging-guidelines/Rust/#_replacing_git_dependencies
%global pubgrub_git https://github.com/astral-sh/pubgrub
%global pubgrub_rev a68cbd1a26e43986a31563e1d127e83bafca3a0c
%global pubgrub_baseversion 0.2.1
%global pubgrub_snapdate 20240619
Source2:        %{pubgrub_git}/archive/%{pubgrub_rev}/pubgrub-%{pubgrub_rev}.tar.gz

# Downstream-only: do not override the default allocator
#
# In https://github.com/astral-sh/uv/pull/399, it was reasonably claimed
# that using tikv-jemallocator improved benchmarks by 10%, so it may be
# worth packaging it and dropping this patch.
Patch:          0001-Downstream-only-do-not-override-the-default-allocato.patch

# Downstream-only: Always find the system-wide uv executable
#
# “Should uv.find_uv_bin() be able to find /usr/bin/uv?”
# https://github.com/astral-sh/uv/issues/4451
Patch:          0001-Downstream-patch-always-find-the-system-wide-uv-exec.patch

# Downstream-only: Revert "feat: ensure successful round-trip of RON (#193)"
#
# This reverts commit 21c6a215432fea9a75b7d15d9a9936af9ccc17cb.
#
# We will not be packaging an alpha version of rust-ron. We can adjust this
# after ron 0.9.x is released.
Patch100:       0001-Downstream-only-Revert-feat-ensure-successful-round-.patch

BuildRequires:  cargo-rpm-macros >= 24
BuildRequires:  rust2rpm-helper
BuildRequires:  tomcli

BuildRequires:  python3-devel

# This is a fork of async_zip; see the notes about Source1.
%global async_zip_snapinfo %{async_zip_snapdate}git%{sub %{async_zip_rev} 1 7}
%global async_zip_version %{async_zip_baseversion}^%{async_zip_snapinfo}
Provides:       bundled(crate(async_zip)) = %{async_zip_version}
# This is a fork of pubgrub; see the notes about Source2.
%global pubgrub_snapinfo %{pubgrub_snapdate}git%{sub %{pubgrub_rev} 1 7}
%global pubgrub_version %{pubgrub_baseversion}^%{pubgrub_snapinfo}
Provides:       bundled(crate(pubgrub)) = %{pubgrub_version}

%global common_description %{expand:
An extremely fast Python package installer and resolver, written in Rust.
Designed as a drop-in replacement for common pip and pip-tools workflows.

Highlights:

  • ⚖️  Drop-in replacement for common pip, pip-tools, and virtualenv commands.
  • ⚡️ 10-100x faster than pip and pip-tools (pip-compile and pip-sync).
  • 💾 Disk-space efficient, with a global cache for dependency deduplication.
  • 🐍 Installable via curl, pip, pipx, etc. uv is a static binary that can be
    installed without Rust or Python.
  • 🧪 Tested at-scale against the top 10,000 PyPI packages.
  • 🖥️ Support for macOS, Linux, and Windows.
  • 🧰 Advanced features such as dependency version overrides and alternative
    resolution strategies.
  • ⁉️  Best-in-class error messages with a conflict-tracking resolver.
  • 🤝 Support for a wide range of advanced pip features, including editable
    installs, Git dependencies, direct URL dependencies, local dependencies,
    constraints, source distributions, HTML and JSON indexes, and more.}

%description %{common_description}


%package -n python3-uv
Summary:        %{summary}

BuildArch:      noarch

Requires:       uv = %{version}-%{release}

%description -n python3-uv %{common_description}

This package provides an importable Python module for uv.


%prep
%autosetup -N
%autopatch -p1 -M99

# See comments above Source1:
%setup -q -T -D -b 1 -n uv-%{version}
tomcli set Cargo.toml del workspace.dependencies.async_zip.git
tomcli set Cargo.toml del workspace.dependencies.async_zip.rev
# Adding the crate to the workspace (in this case implicitly, by linking it
# under crates/) means %%cargo_generate_buildrequires can handle it correctly.
ln -s '../../rs-async-zip-%{async_zip_rev}' crates/async_zip
tomcli set Cargo.toml str workspace.dependencies.async_zip.path \
    'crates/async_zip'
# Drop dev-dependencies that are only required for compiling the
# actix-multipart example, and remove it. Note that while “futures” is in this
# section of Cargo.toml, it is in fact also used in a test.
tomcli set crates/async_zip/Cargo.toml del dev-dependencies.actix-web
tomcli set crates/async_zip/Cargo.toml del dev-dependencies.actix-multipart
tomcli set crates/async_zip/Cargo.toml del dev-dependencies.derive_more
tomcli set crates/async_zip/Cargo.toml del dev-dependencies.uuid
mv crates/async_zip/examples/actix_multipart.rs{,.disabled}

# See comments above Source2:
%setup -q -T -D -b 2 -n uv-%{version}
tomcli set Cargo.toml del workspace.dependencies.pubgrub.git
tomcli set Cargo.toml del workspace.dependencies.pubgrub.rev
ln -s '../../pubgrub-%{pubgrub_rev}' crates/pubgrub
pushd crates/pubgrub
%patch -p1 100
popd
tomcli set Cargo.toml str workspace.dependencies.pubgrub.path \
    'crates/pubgrub'

# Patch out foreign (e.g. Windows-only) dependencies.
find . -type f -name Cargo.toml -print \
    -execdir rust2rpm-helper strip-foreign -o '{}' '{}' ';'

# Do not strip the compiled Python module; we need useful debuginfo. Upstream
# set this intentionally, so this makes sense to keep downstream-only.
tomcli set pyproject.toml false tool.maturin.strip

# Drop a benchmark-only dev-dependency.
tomcli set crates/pubgrub/Cargo.toml del dev-dependencies.criterion
# Exclude the bench crate from the workspace. We don’t need to build and run
# benchmarks, and it brings in unwanted additional dev dependencies.
tomcli set Cargo.toml append workspace.exclude crates/bench
# The uv-dev crate provides “development utilities for uv,” which should not be
# needed here, and it also brings in extra dependencies that we would prefer to
# do without.
tomcli set Cargo.toml append workspace.exclude crates/uv-dev

# The pypi feature, described as “Introduces a dependency on PyPI,” in practice
# controls whether we build and run tests that want to talk to PyPI. In an
# offline build, we definitely don’t want that, so we remove it from the
# default features.
tomcli set crates/uv/Cargo.toml lists delitem features.default 'pypi'

# Omit some tests that are not gated with the pypi feature, but require
# multiple specific Python interpreter versions (down to patch release number),
# which upstream normally downloads, precompiled, into the build area.
mv crates/uv/tests/pip_check.rs{,.disabled}
mv crates/uv/tests/pip_list.rs{,.disabled}
mv crates/uv/tests/pip_show.rs{,.disabled}
mv crates/uv/tests/pip_uninstall.rs{,.disabled}
mv crates/uv/tests/venv.rs{,.disabled}
mv crates/uv/tests/workspace.rs{,.disabled}

# Omit tests requiring wiremock; its dependency tree is too large and complex
# to consider packaging it right now. The conditional #[cfg(any())] is always
# false.
sed -r -i 's/^#\[cfg\(test\)\]/#[cfg(any())]\r&/' \
    crates/uv-auth/src/middleware.rs
tomcli set crates/uv-auth/Cargo.toml del dev-dependencies.wiremock

# Omit tests requiring varisat; it is not packaged and has significant
# dependencies of its own.
tomcli set crates/pubgrub/Cargo.toml del dev-dependencies.varisat
mv crates/pubgrub/tests/proptest.rs{,.disabled}
mv crates/pubgrub/tests/sat_dependency_provider.rs{,.disabled}

%if 0%{?__isa_bits} == 32
# These tests check zip64 support with files >4GiB, which doesn’t work on
# 32-bit platforms because the file size doesn’t fit in a usize.
mv crates/async_zip/src/tests/write/zip64/mod.rs{,.disabled}
sed -r -i 's@^mod zip64;@// &@' crates/async_zip/src/tests/write/mod.rs
%endif

# For unclear reasons, maturin checks for the presence of optional crate
# dependencies that correspond to features we have not enabled. We need to
# patch out those that are not packaged, an unfortunate but straightforward
# hack. See further commentary in %%generate_buildrequires.
tomcli set crates/uv/Cargo.toml del dependencies.axoupdater
tomcli set crates/uv/Cargo.toml del features.self-update
tomcli set crates/uv/Cargo.toml del dependencies.tracing-durations-export

# Loosen some version bounds that were aggressively updated upstream by the
# renovate bot. We retain this comment and the following example even when
# there are currently no dependencies that need to be adjusted.
#
# # foocrate
# #   wanted: 0.2.0
# #   currently packaged: 0.1.2
# #   https://bugzilla.redhat.com/show_bug.cgi?id=1234567
# tomcli set crates/uv/Cargo.toml str dev-dependencies.foocrate.version 0.1.2

# mailparse
#   wanted: 0.15.0
#   currently packaged: 0.14.0
#   https://bugzilla.redhat.com/show_bug.cgi?id=2258714
tomcli set Cargo.toml str \
    workspace.dependencies.mailparse.version '>=0.14,<0.16'

%cargo_prep


%generate_buildrequires
# For unclear reasons, maturin checks for all crate dependencies when it is
# invoked as part of %%pyproject_buildrequires – including those corresponding
# to optional features.
#
# Furthermore, if we do not supply -a to %%cargo_generate_buildrequires, then
# maturin will fail looking for crates like pyo3 (and will still look for
# optional crate dependencies).
#
# Since maturin always checks for dev-dependencies, we need -t so that they are
# generated even when the “check” bcond is disabled.
%cargo_generate_buildrequires -a -t
%pyproject_buildrequires


%build
export RUSTFLAGS='%{build_rustflags}'
%pyproject_wheel

%{cargo_license_summary}
%{cargo_license} > LICENSE.dependencies


%install
%pyproject_install
%pyproject_save_files uv

if [ '%{python3_sitearch}' != '%{python3_sitelib}' ]
then
  # Maturin is really designed to build compiled Python extensions, but (when
  # the uv executable is not bundled in the Python package) the uv Python
  # library is actually pure-Python, and the python3-uv subpackage can be
  # noarch. We can’t tell maturin to install to the appropriate site-packages
  # directory, but we can fix the installation path manually.
  install -d %{buildroot}%{python3_sitelib}
  mv %{buildroot}%{python3_sitearch}/uv* %{buildroot}%{python3_sitelib}
  sed -r -i 's@%{python3_sitearch}@%{python3_sitelib}@' %{pyproject_files}
fi

# generate and install shell completions
target/rpm/uv --generate-shell-completion bash > uv.bash
target/rpm/uv --generate-shell-completion fish > uv.fish
target/rpm/uv --generate-shell-completion zsh > _uv
 
install -Dpm 0644 uv.bash -t %{buildroot}/%{bash_completions_dir}
install -Dpm 0644 uv.fish -t %{buildroot}/%{fish_completions_dir}
install -Dpm 0644 _uv -t %{buildroot}/%{zsh_completions_dir}


%check
%if %{with check}
# These tests rely on debug assertions, and fail when tests are compiled in
# release mode:
#
# -p uv-auth --lib:
skip="${skip-} --skip keyring::test::fetch_url_no_host"
skip="${skip-} --skip keyring::test::fetch_url_with_no_username"
skip="${skip-} --skip keyring::test::fetch_url_with_password"
#
# These tests require network access:
#
# -p uv-client --test remote_metadata
skip="${skip-} --skip remote_metadata_with_and_without_cache"

%cargo_test -- -- --exact ${skip-}
%endif

%pyproject_check_import


%files
%license LICENSE-APACHE LICENSE-MIT LICENSE.dependencies
%doc CHANGELOG.md
%doc README.md
%doc PIP_COMPATIBILITY.md
%doc PREVIEW-CHANGELOG.md

%{_bindir}/uv

%{bash_completions_dir}/uv.bash
%{fish_completions_dir}/uv.fish
%{zsh_completions_dir}/_uv


%files -n python3-uv -f %{pyproject_files}


%changelog
%autochangelog
